@@ -1,7 +1,5 @@
 //! Provides build phase specific types and helpers.
 
-use std::path::PathBuf;
-
 use crate::buildpack::Buildpack;
 use crate::data::buildpack::StackId;
 use crate::data::layer::LayerName;
@@ -10,6 +8,7 @@ use crate::data::{
     buildpack::SingleBuildpackDescriptor, buildpack_plan::BuildpackPlan, launch::Launch,
 };
 use crate::layer::{HandleLayerErrorOrBuildpackError, Layer, LayerData};
+use std::path::PathBuf;
 
 /// Context for the build phase execution.
 pub struct BuildContext<B: Buildpack + ?Sized> {
@@ -118,6 +117,7 @@ impl<B: Buildpack + ?Sized> BuildContext<B> {
 ///
 /// To construct values of this type, use a [`BuildResultBuilder`].
 #[derive(Debug)]
+#[must_use]
 pub struct BuildResult(pub(crate) InnerBuildResult);
 
 #[derive(Debug)]
@@ -143,18 +143,16 @@ pub(crate) enum InnerBuildResult {
 ///    .launch(Launch::new().process(ProcessBuilder::new(process_type!("type"), "command").arg("-v").build()))
 ///    .build();
 /// ```
+#[derive(Default)]
+#[must_use]
 pub struct BuildResultBuilder {
     launch: Option<Launch>,
     store: Option<Store>,
 }
 
 impl BuildResultBuilder {
-    #[must_use]
     pub fn new() -> Self {
-        Self {
-            launch: None,
-            store: None,
-        }
+        Self::default()
     }
 
     /// Builds the final [`BuildResult`].
@@ -168,7 +166,6 @@ impl BuildResultBuilder {
         Ok(self.build_unwrapped())
     }
 
-    #[must_use]
     pub fn build_unwrapped(self) -> BuildResult {
         BuildResult(InnerBuildResult::Pass {
             launch: self.launch,
@@ -176,21 +173,13 @@ impl BuildResultBuilder {
         })
     }
 
-    #[must_use]
     pub fn launch(mut self, launch: Launch) -> Self {
         self.launch = Some(launch);
         self
     }
 
-    #[must_use]
     pub fn store(mut self, store: Store) -> Self {
         self.store = Some(store);
         self
     }
 }
-
-impl Default for BuildResultBuilder {
-    fn default() -> Self {
-        Self::new()
-    }
-}
