@@ -1,7 +1,6 @@
 use crate::build::BuildContext;
 use crate::data::layer::LayerName;
 use crate::data::layer_content_metadata::LayerContentMetadata;
-
 use crate::generic::GenericMetadata;
 use crate::layer::{ExistingLayerStrategy, Layer, LayerData, MetadataMigration};
 use crate::layer_env::LayerEnv;
@@ -10,8 +9,9 @@ use crate::Buildpack;
 use crate::{write_toml_file, TomlFileError};
 use serde::de::DeserializeOwned;
 use serde::Serialize;
+use std::collections::HashMap;
 use std::fs;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 
 pub(crate) fn handle_layer<B: Buildpack + ?Sized, L: Layer<Buildpack = B>>(
     context: &BuildContext<B>,
@@ -45,6 +45,7 @@ pub(crate) fn handle_layer<B: Buildpack + ?Sized, L: Layer<Buildpack = B>>(
                             types: Some(layer.types()),
                             metadata: layer_data.content_metadata.metadata,
                         },
+                        ExecDPrograms::Keep,
                     )?;
 
                     // Reread the layer from disk to ensure the returned layer data accurately reflects
@@ -78,6 +79,7 @@ pub(crate) fn handle_layer<B: Buildpack + ?Sized, L: Layer<Buildpack = B>>(
                                     types: generic_layer_data.content_metadata.types,
                                     metadata: migrated_metadata,
                                 },
+                                ExecDPrograms::Keep,
                             )?;
                         }
                     }
@@ -117,6 +119,7 @@ fn handle_create_layer<B: Buildpack + ?Sized, L: Layer<Buildpack = B>>(
             types: Some(layer.types()),
             metadata: layer_result.metadata,
         },
+        ExecDPrograms::Overwrite(layer_result.exec_d_programs),
     )?;
 
     read_layer(&context.layers_dir, layer_name)?
@@ -141,6 +144,7 @@ fn handle_update_layer<B: Buildpack + ?Sized, L: Layer<Buildpack = B>>(
             types: Some(layer.types()),
             metadata: layer_result.metadata,
         },
+        ExecDPrograms::Overwrite(layer_result.exec_d_programs),
     )?;
 
     read_layer(&context.layers_dir, &layer_data.name)?
@@ -211,15 +215,19 @@ pub enum WriteLayerError {
     TomlFileError(#[from] TomlFileError),
 }
 
+#[derive(Debug)]
+enum ExecDPrograms {
+    Keep,
+    Overwrite(HashMap<String, PathBuf>),
+}
+
 /// Does not error if the layer doesn't exist.
 fn delete_layer<P: AsRef<Path>>(
     layers_dir: P,
     layer_name: &LayerName,
 ) -> Result<(), std::io::Error> {
     let layer_dir = layers_dir.as_ref().join(layer_name.as_str());
-    let layer_toml = layers_dir
-        .as_ref()
-        .join(format!("{}.toml", layer_name.as_ref()));
+    let layer_toml = layers_dir.as_ref().join(format!("{layer_name}.toml"));
 
     default_on_not_found(fs::remove_dir_all(&layer_dir))?;
     default_on_not_found(fs::remove_file(&layer_toml))?;
@@ -233,16 +241,34 @@ fn write_layer<M: Serialize, P: AsRef<Path>>(
     layer_name: &LayerName,
     layer_env: &LayerEnv,
     layer_content_metadata: &LayerContentMetadata<M>,
+    layer_exec_d_programs: ExecDPrograms,
 ) -> Result<(), WriteLayerError> {
     let layer_dir = layers_dir.as_ref().join(layer_name.as_str());
-    let layer_content_metadata_path = layers_dir
-        .as_ref()
-        .join(format!("{}.toml", layer_name.as_ref()));
+    let layer_content_metadata_path = layers_dir.as_ref().join(format!("{layer_name}.toml"));
 
     fs::create_dir_all(&layer_dir)?;
     layer_env.write_to_layer_dir(&layer_dir)?;
     write_toml_file(&layer_content_metadata, layer_content_metadata_path)?;
 
+    match layer_exec_d_programs {
+        ExecDPrograms::Overwrite(exec_d_programs) => {
+            let exec_d_dir = layer_dir.join("exec.d");
+
+            if exec_d_dir.is_dir() {
+                fs::remove_dir_all(&exec_d_dir)?;
+            }
+
+            if !exec_d_programs.is_empty() {
+                fs::create_dir_all(&exec_d_dir)?;
+
+                for (name, path) in exec_d_programs {
+                    fs::copy(path, exec_d_dir.join(name))?;
+                }
+            }
+        }
+        ExecDPrograms::Keep => {}
+    }
+
     Ok(())
 }
 
@@ -251,7 +277,7 @@ fn read_layer<M: DeserializeOwned, P: AsRef<Path>>(
     layer_name: &LayerName,
 ) -> Result<Option<LayerData<M>>, ReadLayerError> {
     let layer_dir_path = layers_dir.as_ref().join(layer_name.as_str());
-    let layer_toml_path = layers_dir.as_ref().join(format!("{}.toml", layer_name));
+    let layer_toml_path = layers_dir.as_ref().join(format!("{layer_name}.toml"));
 
     if !layer_dir_path.exists() && !layer_toml_path.exists() {
         return Ok(None);
@@ -315,7 +341,7 @@ mod tests {
 
         fs::create_dir_all(&layer_dir).unwrap();
         fs::write(
-            layers_dir.join(format!("{}.toml", &layer_name)),
+            layers_dir.join(format!("{layer_name}.toml")),
             r#"
             [types]
             launch = true
@@ -328,7 +354,7 @@ mod tests {
         super::delete_layer(&layers_dir, &layer_name).unwrap();
 
         assert!(!layer_dir.exists());
-        assert!(!layers_dir.join(format!("{}.toml", &layer_name)).exists());
+        assert!(!layers_dir.join(format!("{layer_name}.toml")).exists());
     }
 
     #[test]
@@ -339,7 +365,7 @@ mod tests {
         let layer_dir = layers_dir.join(layer_name.as_str());
 
         fs::write(
-            layers_dir.join(format!("{}.toml", &layer_name)),
+            layers_dir.join(format!("{layer_name}.toml")),
             r#"
             [types]
             launch = true
@@ -352,7 +378,7 @@ mod tests {
         super::delete_layer(&layers_dir, &layer_name).unwrap();
 
         assert!(!layer_dir.exists());
-        assert!(!layers_dir.join(format!("{}.toml", &layer_name)).exists());
+        assert!(!layers_dir.join(format!("{layer_name}.toml")).exists());
     }
 
     #[test]
@@ -371,6 +397,10 @@ mod tests {
         let layers_dir = temp_dir.path();
         let layer_dir = layers_dir.join(layer_name.as_str());
 
+        let execd_source_temp_dir = tempdir().unwrap();
+        let foo_execd_file = execd_source_temp_dir.path().join("foo");
+        fs::write(&foo_execd_file, "foo-contents").unwrap();
+
         super::write_layer(
             &layers_dir,
             &layer_name,
@@ -388,6 +418,7 @@ mod tests {
                 }),
                 metadata: GenericMetadata::default(),
             },
+            ExecDPrograms::Overwrite(HashMap::from([(String::from("foo"), foo_execd_file)])),
         )
         .unwrap();
 
@@ -398,8 +429,13 @@ mod tests {
             "ENV_VAR_VALUE"
         );
 
+        assert_eq!(
+            fs::read_to_string(layer_dir.join("exec.d/foo")).unwrap(),
+            "foo-contents"
+        );
+
         let layer_content_metadata: LayerContentMetadata<GenericMetadata> =
-            read_toml_file(layers_dir.join(format!("{}.toml", layer_name))).unwrap();
+            read_toml_file(layers_dir.join(format!("{layer_name}.toml"))).unwrap();
 
         assert_eq!(
             layer_content_metadata.types,
@@ -418,6 +454,14 @@ mod tests {
         let layers_dir = temp_dir.path();
         let layer_dir = layers_dir.join(layer_name.as_str());
 
+        let execd_source_temp_dir = tempdir().unwrap();
+        let foo_execd_file = execd_source_temp_dir.path().join("foo");
+        let bar_execd_file = execd_source_temp_dir.path().join("bar");
+        let baz_execd_file = execd_source_temp_dir.path().join("baz");
+        fs::write(&foo_execd_file, "foo-contents").unwrap();
+        fs::write(&bar_execd_file, "bar-contents").unwrap();
+        fs::write(&baz_execd_file, "baz-contents").unwrap();
+
         super::write_layer(
             &layers_dir,
             &layer_name,
@@ -442,6 +486,7 @@ mod tests {
                 }),
                 metadata: GenericMetadata::default(),
             },
+            ExecDPrograms::Overwrite(HashMap::from([(String::from("foo"), foo_execd_file)])),
         )
         .unwrap();
 
@@ -464,6 +509,10 @@ mod tests {
                 }),
                 metadata: GenericMetadata::default(),
             },
+            ExecDPrograms::Overwrite(HashMap::from([
+                (String::from("bar"), bar_execd_file),
+                (String::from("baz"), baz_execd_file),
+            ])),
         )
         .unwrap();
 
@@ -481,8 +530,20 @@ mod tests {
 
         assert!(!layer_dir.join("env/SOME_OTHER_ENV_VAR.default").exists());
 
+        assert!(!layer_dir.join("exec.d/foo").exists());
+
+        assert_eq!(
+            fs::read_to_string(layer_dir.join("exec.d/bar")).unwrap(),
+            "bar-contents"
+        );
+
+        assert_eq!(
+            fs::read_to_string(layer_dir.join("exec.d/baz")).unwrap(),
+            "baz-contents"
+        );
+
         let layer_content_metadata: LayerContentMetadata<GenericMetadata> =
-            read_toml_file(layers_dir.join(format!("{}.toml", layer_name))).unwrap();
+            read_toml_file(layers_dir.join(format!("{layer_name}.toml"))).unwrap();
 
         assert_eq!(
             layer_content_metadata.types,
@@ -494,6 +555,137 @@ mod tests {
         );
     }
 
+    #[test]
+    fn write_layer_keep_execd() {
+        let layer_name = layer_name!("foo");
+        let temp_dir = tempdir().unwrap();
+        let layers_dir = temp_dir.path();
+        let layer_dir = layers_dir.join(layer_name.as_str());
+
+        super::write_layer(
+            &layers_dir,
+            &layer_name,
+            &LayerEnv::new(),
+            &LayerContentMetadata {
+                types: Some(LayerTypes {
+                    launch: false,
+                    build: false,
+                    cache: true,
+                }),
+                metadata: GenericMetadata::default(),
+            },
+            ExecDPrograms::Keep,
+        )
+        .unwrap();
+
+        assert!(!layer_dir.join("exec.d").exists());
+    }
+
+    #[test]
+    fn write_existing_layer_keep_execd() {
+        let layer_name = layer_name!("foo");
+        let temp_dir = tempdir().unwrap();
+        let layers_dir = temp_dir.path();
+        let layer_dir = layers_dir.join(layer_name.as_str());
+
+        let execd_source_temp_dir = tempdir().unwrap();
+        let foo_execd_file = execd_source_temp_dir.path().join("foo");
+        fs::write(&foo_execd_file, "foo-contents").unwrap();
+
+        super::write_layer(
+            &layers_dir,
+            &layer_name,
+            &LayerEnv::new(),
+            &LayerContentMetadata {
+                types: Some(LayerTypes {
+                    launch: false,
+                    build: false,
+                    cache: true,
+                }),
+                metadata: GenericMetadata::default(),
+            },
+            ExecDPrograms::Overwrite(HashMap::from([(String::from("foo"), foo_execd_file)])),
+        )
+        .unwrap();
+
+        assert_eq!(
+            fs::read_to_string(layer_dir.join("exec.d/foo")).unwrap(),
+            "foo-contents"
+        );
+
+        super::write_layer(
+            &layers_dir,
+            &layer_name,
+            &LayerEnv::new(),
+            &LayerContentMetadata {
+                types: Some(LayerTypes {
+                    launch: false,
+                    build: false,
+                    cache: true,
+                }),
+                metadata: GenericMetadata::default(),
+            },
+            ExecDPrograms::Keep,
+        )
+        .unwrap();
+
+        assert_eq!(
+            fs::read_to_string(layer_dir.join("exec.d/foo")).unwrap(),
+            "foo-contents"
+        );
+    }
+
+    #[test]
+    fn write_existing_layer_overwrite_with_empty_execd() {
+        let layer_name = layer_name!("foo");
+        let temp_dir = tempdir().unwrap();
+        let layers_dir = temp_dir.path();
+        let layer_dir = layers_dir.join(layer_name.as_str());
+
+        let execd_source_temp_dir = tempdir().unwrap();
+        let foo_execd_file = execd_source_temp_dir.path().join("foo");
+        fs::write(&foo_execd_file, "foo-contents").unwrap();
+
+        super::write_layer(
+            &layers_dir,
+            &layer_name,
+            &LayerEnv::new(),
+            &LayerContentMetadata {
+                types: Some(LayerTypes {
+                    launch: false,
+                    build: false,
+                    cache: true,
+                }),
+                metadata: GenericMetadata::default(),
+            },
+            ExecDPrograms::Overwrite(HashMap::from([(String::from("foo"), foo_execd_file)])),
+        )
+        .unwrap();
+
+        assert_eq!(
+            fs::read_to_string(layer_dir.join("exec.d/foo")).unwrap(),
+            "foo-contents"
+        );
+
+        super::write_layer(
+            &layers_dir,
+            &layer_name,
+            &LayerEnv::new(),
+            &LayerContentMetadata {
+                types: Some(LayerTypes {
+                    launch: false,
+                    build: false,
+                    cache: true,
+                }),
+                metadata: GenericMetadata::default(),
+            },
+            ExecDPrograms::Overwrite(HashMap::new()),
+        )
+        .unwrap();
+
+        assert!(!layer_dir.join("exec.d").exists());
+    }
+
     #[test]
     fn read_layer() {
         #[derive(Deserialize, Debug, Eq, PartialEq)]
@@ -509,7 +701,7 @@ mod tests {
 
         fs::create_dir_all(&layer_dir).unwrap();
         fs::write(
-            layers_dir.join(format!("{}.toml", &layer_name)),
+            layers_dir.join(format!("{layer_name}.toml")),
             r#"
             [types]
             launch = true
@@ -578,7 +770,7 @@ mod tests {
 
         fs::create_dir_all(&layer_dir).unwrap();
         fs::write(
-            layers_dir.join(format!("{}.toml", &layer_name)),
+            layers_dir.join(format!("{layer_name}.toml")),
             r#"
             [types
             build = true
@@ -611,7 +803,7 @@ mod tests {
 
         fs::create_dir_all(&layer_dir).unwrap();
         fs::write(
-            layers_dir.join(format!("{}.toml", &layer_name)),
+            layers_dir.join(format!("{layer_name}.toml")),
             r#"
             [types]
             build = true
@@ -661,7 +853,7 @@ mod tests {
         let temp_dir = tempdir().unwrap();
         let layers_dir = temp_dir.path();
 
-        fs::write(layers_dir.join(format!("{}.toml", &layer_name)), "").unwrap();
+        fs::write(layers_dir.join(format!("{layer_name}.toml")), "").unwrap();
 
         match super::read_layer::<GenericMetadata, _>(&layers_dir, &layer_name) {
             Ok(None) => {}
