@@ -6,6 +6,7 @@ use crate::layer_env::LayerEnv;
 use crate::Buildpack;
 use serde::de::DeserializeOwned;
 use serde::Serialize;
+use std::collections::HashMap;
 use std::path::{Path, PathBuf};
 
 /// Represents a buildpack layer written with the libcnb framework.
@@ -166,12 +167,14 @@ pub struct LayerData<M> {
 pub struct LayerResult<M> {
     pub metadata: M,
     pub env: Option<LayerEnv>,
+    pub exec_d_programs: HashMap<String, PathBuf>,
 }
 
 /// A builder that simplifies the creation of [`LayerResult`] values.
 pub struct LayerResultBuilder<M> {
     metadata: M,
     env: Option<LayerEnv>,
+    exec_d_programs: HashMap<String, PathBuf>,
 }
 
 impl<M> LayerResultBuilder<M> {
@@ -180,6 +183,7 @@ impl<M> LayerResultBuilder<M> {
         Self {
             metadata,
             env: None,
+            exec_d_programs: HashMap::new(),
         }
     }
 
@@ -189,6 +193,34 @@ impl<M> LayerResultBuilder<M> {
         self
     }
 
+    /// Adds an exec.d program to the layer.
+    ///
+    /// # Example
+    ///
+    /// ```compile_fail
+    /// use libcnb::generic::GenericMetadata;
+    /// use libcnb::layer::LayerResultBuilder;
+    /// use libcnb::additional_buildpack_binary_path;
+    ///
+    /// LayerResultBuilder::new(GenericMetadata::default())
+    ///        .exec_d_program(
+    ///            "program_name_in_layer",
+    ///            // This does not compile in this doctest since there is no binary target with this name.
+    ///            additional_buildpack_binary_path!("binary_target_name"),
+    ///        )
+    ///        .build();
+    /// ```
+    #[must_use]
+    pub fn exec_d_program(
+        mut self,
+        name: impl Into<String>,
+        exec_d_program: impl Into<PathBuf>,
+    ) -> Self {
+        self.exec_d_programs
+            .insert(name.into(), exec_d_program.into());
+        self
+    }
+
     /// Builds the final [`LayerResult`].
     ///
     /// This method returns the [`LayerResult`] wrapped in a [`Result`] even though its technically
@@ -206,6 +238,7 @@ impl<M> LayerResultBuilder<M> {
         LayerResult {
             metadata: self.metadata,
             env: self.env,
+            exec_d_programs: self.exec_d_programs,
         }
     }
 }
