@@ -1,5 +1,6 @@
 //! Type-safe, in-memory, layer environment variables.
 
+use crate::Env;
 use std::cmp::Ordering;
 use std::collections::hash_map::Entry;
 use std::collections::{BTreeMap, HashMap};
@@ -7,8 +8,6 @@ use std::ffi::OsString;
 use std::fs;
 use std::path::Path;
 
-use crate::Env;
-
 /// Represents environment variable modifications of a Cloud Native Buildpack layer.
 ///
 /// Cloud Native Buildpacks can add a special directory to their layer directories to modify the
@@ -80,7 +79,7 @@ use crate::Env;
 /// assert_eq!(env.get("PATH").unwrap(), layer_dir.join("bin"));
 /// assert_eq!(env.get("CPATH"), None); // None, because CPATH is only added during build
 /// ```
-#[derive(Eq, PartialEq, Debug, Clone)]
+#[derive(Eq, PartialEq, Debug, Default, Clone)]
 pub struct LayerEnv {
     all: LayerEnvDelta,
     build: LayerEnvDelta,
@@ -113,14 +112,7 @@ impl LayerEnv {
     /// ```
     #[must_use]
     pub fn new() -> Self {
-        Self {
-            all: LayerEnvDelta::new(),
-            build: LayerEnvDelta::new(),
-            launch: LayerEnvDelta::new(),
-            process: HashMap::new(),
-            layer_paths_build: LayerEnvDelta::new(),
-            layer_paths_launch: LayerEnvDelta::new(),
-        }
+        Self::default()
     }
 
     /// Applies this [`LayerEnv`] to the given [`Env`] for the given [`Scope`].
@@ -382,12 +374,6 @@ impl LayerEnv {
     }
 }
 
-impl Default for LayerEnv {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
 /// Environment variable modification behavior.
 /// ([CNB spec: Environment Variable Modification Rules](https://github.com/buildpacks/spec/blob/main/buildpack.md#environment-variable-modification-rules))
 #[derive(Eq, PartialEq, Debug, Clone)]
@@ -432,16 +418,14 @@ pub enum Scope {
     Process(String),
 }
 
-#[derive(Eq, PartialEq, Debug, Clone)]
+#[derive(Eq, PartialEq, Debug, Default, Clone)]
 struct LayerEnvDelta {
     entries: BTreeMap<(ModificationBehavior, OsString), OsString>,
 }
 
 impl LayerEnvDelta {
     fn new() -> Self {
-        Self {
-            entries: BTreeMap::new(),
-        }
+        Self::default()
     }
 
     fn apply(&self, env: &Env) -> Env {
