@@ -14,10 +14,15 @@
 
 pub mod build;
 pub mod detect;
+pub mod exec_d;
 pub mod generic;
 pub mod layer;
 pub mod layer_env;
 
+// Internals that need to be public for macros
+#[doc(hidden)]
+pub mod internals;
+
 mod buildpack;
 mod env;
 mod error;
@@ -26,16 +31,14 @@ mod runtime;
 mod toml_file;
 mod util;
 
-#[doc(inline)]
-pub use libcnb_data as data;
-
+pub use buildpack::Buildpack;
 pub use env::*;
 pub use error::*;
+#[doc(inline)]
+pub use libcnb_data as data;
 pub use platform::*;
-pub use toml_file::*;
-
-pub use buildpack::Buildpack;
 pub use runtime::libcnb_runtime;
+pub use toml_file::*;
 
 const LIBCNB_SUPPORTED_BUILDPACK_API: data::buildpack::BuildpackApi =
     data::buildpack::BuildpackApi { major: 0, minor: 6 };
@@ -78,6 +81,50 @@ macro_rules! buildpack_main {
     };
 }
 
+/// Resolves the path to an additional buildpack binary by Cargo target name.
+///
+/// This can be used to copy additional binaries to layers or use them for exec.d.
+///
+/// To add an additional binary to a buildpack, add a new file with a main function to `bin/`.
+/// Cargo will [automatically configure it as a binary target](https://doc.rust-lang.org/cargo/reference/cargo-targets.html#target-auto-discovery)
+/// with the name of file.
+///
+/// **Note**: This only works properly if the buildpack is packaged with `libcnb-cargo`/`libcnb-test`.
+///
+/// ```no_run,compile_fail
+/// use libcnb::additional_buildpack_binary_path;
+/// # let layer_dir = std::path::PathBuf::from(".");
+///
+/// std::fs::copy(
+///     // This would not compile in this doctest since there is no `runtime_tool` binary target.
+///     additional_buildpack_binary_path!("runtime_tool"),
+///     layer_dir.join("runtime_tool"),
+/// )
+/// .unwrap();
+/// ```
+#[macro_export]
+macro_rules! additional_buildpack_binary_path {
+    ($target_name:expr) => {
+        ::libcnb::internals::verify_bin_target_exists!(
+            $target_name,
+            {
+                ::std::env::var("CNB_BUILDPACK_DIR")
+                    .map(::std::path::PathBuf::from)
+                    .expect("Could not read CNB_BUILDPACK_DIR environment variable")
+                    .join(".libcnb-cargo")
+                    .join("additional-bin")
+                    .join($target_name)
+            },
+            {
+                compile_error!(concat!(
+                    $target_name,
+                    " is not a valid binary target in this buildpack crate!"
+                ))
+            }
+        )
+    };
+}
+
 // This runs the README.md as a doctest, ensuring the code examples in it are valid.
 // It will not be part of the final crate.
 #[cfg(doctest)]
